package org.hardknockjava.lesson27.driver;

import java.awt.Color;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.imageio.ImageIO;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDPageContentStream;
import org.apache.pdfbox.pdmodel.common.PDRectangle;
import org.apache.pdfbox.pdmodel.font.PDFont;
import org.apache.pdfbox.pdmodel.font.PDType1Font;
import org.apache.pdfbox.pdmodel.font.Standard14Fonts;
import org.apache.pdfbox.pdmodel.graphics.image.PDImageXObject;

import data.GettysburgAddress;

public class GettysburgPdfWriter {
    private GettysburgAddress address;
    private float margin = 72;

    private static final float IMAGE_WIDTH_IN_POINTS = 108;

    public static void main(String[] args) throws Exception {
        GettysburgAddress address = new GettysburgAddress();
        GettysburgPdfWriter printer = new GettysburgPdfWriter(address);
        printer.printAddress();
        System.out.println("Done");
    }

    public GettysburgPdfWriter(GettysburgAddress address) {
        this.address = address;
    }

    public void printAddress() throws Exception {
        PDDocument doc = new PDDocument();
        printAddress(doc);

        Date today = new Date();
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd-HH24mmss");
        String fileName = "address-" + format.format(today) + ".pdf";
        File addressDir = new File("pdf");
        addressDir.mkdirs();
        File addressFile = new File(addressDir, fileName);
        addressFile.delete();
        doc.save(addressFile);
        doc.close();
    }

    private void printAddress(PDDocument doc) throws Exception {
        PDPage page1 = new PDPage();
        doc.addPage(page1);
        PDPageContentStream contentStream1 = new PDPageContentStream(doc, page1);
        try {
            printPage1(doc, page1, contentStream1);
        } finally {
            contentStream1.close();
        }

        PDPage page2 = new PDPage();
        doc.addPage(page2);
        PDPageContentStream contentStream2 = new PDPageContentStream(doc, page2);
        printPage2(doc, page2, contentStream2);
    }

    private void printPage1(PDDocument doc, PDPage page, PDPageContentStream stream) throws Exception {
        // Add the photograph to the page. Scale it so
        // its width is IMAGE_WIDTH.
        float heightToWidth = ((float) address.getImage().getHeight(null))
                / ((float) (address.getImage().getWidth(null)));
        float targetImageHeight = IMAGE_WIDTH_IN_POINTS * heightToWidth;
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        ImageIO.write(address.getImage(), "jpg", os);
        byte[] imageArray = os.toByteArray();
        PDImageXObject pdImage = PDImageXObject.createFromByteArray(doc, imageArray, "lincoln");

        // The starting y coordinate is the height of the print
        // area less originY.
        PDRectangle box = page.getMediaBox();
        float imageOriginY = (box.getHeight() - margin - targetImageHeight);
        stream.drawImage(pdImage, margin, imageOriginY, IMAGE_WIDTH_IN_POINTS, targetImageHeight);

        // Compute where the first line of text will begin horizontally.
        float fontSize = 14F;
        float lineHeight = fontSize + 2;
        float x = margin + IMAGE_WIDTH_IN_POINTS + 6;
        float y = imageOriginY + targetImageHeight - lineHeight;

        PDFont font = new PDType1Font(Standard14Fonts.FontName.TIMES_ROMAN);
        stream.setFont(font, fontSize);
        stream.setLeading(lineHeight);

        // Get the words of the text and convert the array to a List.
        String[] words = address.getWords();
        List<String> wordList = new ArrayList<String>(Arrays.asList(words));

        // As long as there are words in the list, build a buffer
        // until the next word exceeds the maximum on the current
        // line. Then print the buffer as is, determine the next
        // y and x, and restart.
        StringBuilder buf = new StringBuilder();
        double textLimit = box.getWidth() - x - margin;

        stream.beginText();
        stream.newLineAtOffset(x, y + lineHeight);
        while (!wordList.isEmpty()) {
            String nextWord = wordList.remove(0);
            if (textWidth(buf.toString() + " " + nextWord, font, fontSize) > textLimit) {
                stream.newLine();
                stream.showText(buf.toString());
                buf = new StringBuilder();

                y -= lineHeight;
                float newX;

                // Determine what the new x value should be. If y is still within
                // the height of the image, x remains to the right of the image.
                // Otherwise, x is now at the left margin.
                if (y < (imageOriginY - lineHeight)) {
                    newX = margin;
                } else {
                    newX = margin + IMAGE_WIDTH_IN_POINTS + 6;
                }

                // If the new x differs from the old one, end the text
                // and restart at the left margin, and one line up the
                // page because the next newLine will otherwise leave
                // a gap.
                if (newX != x) {
                    stream.endText();
                    stream.beginText();
                    stream.newLineAtOffset(newX, y + lineHeight);
                    x = newX;
                }
                textLimit = box.getWidth() - x - margin;
            }
            buf.append(nextWord + " ");
        }

        // Start a new line and write the buffer.
        stream.newLine();
        stream.showText(buf.toString());
        stream.endText();
    }

    private float textWidth(String text, PDFont font, float fontSize) throws Exception {
        return font.getStringWidth(text.toString()) / 1000 * fontSize;
    }

    private void printPage2(PDDocument doc, PDPage page, PDPageContentStream stream) throws Exception {
        PDRectangle box = page.getMediaBox();
        float x = box.getLowerLeftX() + 72;
        float y = box.getUpperRightY() - 72;

        // Draw a 1-inch blue line.
        stream.setStrokingColor(Color.BLUE);
        stream.moveTo(x, y);
        stream.lineTo(x + 72, y);
        stream.stroke();

        // Draw a 2-inch square with a red border and yellow fill.
        y = y - 72;
        stream.setStrokingColor(Color.RED);
        stream.setNonStrokingColor(Color.YELLOW);
        stream.addRect(x, y - 144, 144, 144);
        stream.fillAndStroke();
        stream.close();
    }
}
