package org.hardknockjava.lesson27.driver;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import data.GettysburgAddress;

public class GettysburgPrinter implements Printable {
    private static final int DEFAULT_MARGIN = 72;
    private static final int IMAGE_WIDTH_IN_POINTS = 108;
    private GettysburgAddress address;

    public static void main(String[] args) throws Exception {
        GettysburgAddress address = new GettysburgAddress();
        GettysburgPrinter printer = new GettysburgPrinter(address);
        printer.printAddress();

        System.out.println("Done");
    }

    public GettysburgPrinter(GettysburgAddress address) {
        this.address = address;
    }

    public void printAddress() throws Exception {
        PrinterJob job = PrinterJob.getPrinterJob();
        job.setPrintable(this);

        if (!job.printDialog()) {
            return;
        }

        job.print();
    }

    @Override
    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
        // Ensure there's at least a one-inch margin on each edge.
        double margin = pageFormat.getImageableX();
        {
            Paper paper = pageFormat.getPaper();

            if (pageFormat.getImageableX() < DEFAULT_MARGIN || pageFormat.getImageableY() < DEFAULT_MARGIN) {
                margin = DEFAULT_MARGIN;
            }

            paper.setImageableArea(margin, margin, paper.getWidth() - margin * 2, //
                    paper.getHeight() - margin * 2);
            pageFormat.setPaper(paper);
        }

        int imageableX = ((Double) pageFormat.getImageableX()).intValue();
        int imageableY = ((Double) pageFormat.getImageableY()).intValue();
        graphics.translate(imageableX, imageableY);

        switch (pageIndex) {
        case 0: {
            printPage1(graphics, pageFormat);
            return PAGE_EXISTS;
        }
        case 1: {
            printPage2(graphics, pageFormat);
            return PAGE_EXISTS;
        }
        default: {
            return NO_SUCH_PAGE;
        }
        }
    }

    private void printPage1(Graphics graphics, PageFormat pageFormat) {
        // Add the photograph of Lincoln to the page. Scale it so
        // its width is IMAGE_WIDTH.
        float heightToWidth = ((float) address.getImage().getHeight(null)
                / ((float) (address.getImage().getWidth(null))));
        int targetImageHeight = (int) (IMAGE_WIDTH_IN_POINTS * heightToWidth);

        graphics.drawImage(address.getImage(), 0, 0, IMAGE_WIDTH_IN_POINTS, targetImageHeight, null);

        // Compute where the text will begin horizontally.
        int originX = IMAGE_WIDTH_IN_POINTS + 36;
        printText(graphics, pageFormat, originX, targetImageHeight, address);
    }

    private void printText(Graphics graphics, PageFormat pageFormat, float originX, float imageHeight,
            GettysburgAddress address) {
        Font font = new Font(Font.SERIF, Font.PLAIN, 10);
        graphics.setFont(font);
        FontMetrics metrics = graphics.getFontMetrics();

        // Get the line height;
        int lineHeight = metrics.getHeight();

        // Get the words of the text and convert the array to a List.
        String[] words = address.getWords();
        List<String> wordList = new ArrayList<String>(Arrays.asList(words));

        // Establish the starting x and y for the next line.
        int x = (int) (originX);
        int y = 0;

        // As long as there are words in the list, build a buffer
        // until the next word exceeds the maximum on the current
        // line. Then print the buffer as is, determine the next
        // y and x, and restart.
        StringBuilder buf = new StringBuilder();
        double textLimit = pageFormat.getImageableWidth() - x;
        while (!wordList.isEmpty()) {
            String nextWord = wordList.remove(0);
            if (metrics.stringWidth(buf.toString() + " " + nextWord) > textLimit) {
                graphics.drawString(buf.toString(), x, y);
                buf = new StringBuilder();
                y += lineHeight;
                if (y > imageHeight + lineHeight) {
                    x = 0;
                } else {
                    x = (int) (originX + 6);
                }
                textLimit = pageFormat.getImageableWidth() - x;
            }
            buf.append(nextWord + " ");
        }
        graphics.drawString(buf.toString(), x, y);
    }

    private void printPage2(Graphics graphics, PageFormat pageFormat) {
        Graphics2D g2d = (Graphics2D) graphics;

        // Draw a 1-inch blue line.
        graphics.setColor(Color.BLUE);
        graphics.drawLine(0, 72, 72, 72);

        // Draw a 2-inch square with a red border and yellow fill.
        g2d.setColor(Color.YELLOW);
        graphics.fillRect(8, 120, 144, 144);
        graphics.setColor(Color.RED);
        graphics.drawRect(8, 120, 144, 144);

        // Draw a black circle with 1-inch radius and a line width of 6.
        graphics.setColor(Color.BLACK);
        g2d.setStroke(new BasicStroke(6));
        graphics.drawOval(8, 300, 72, 72);
    }
}
